library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity niosII_HomeAutomation is
	port
	(
		-- LCD ports
      LCD_ON  		:	out		std_logic;
      LCD_BLON 	:  out 		std_logic;
      LCD_EN  		:  out 		std_logic;
      LCD_RS  		:  out 		std_logic;
      LCD_RW  		:  out 		std_logic;
      LCD_DATA 	:  inout 	std_logic_vector (7 downto 0);
					  
		-- LEDS
      LEDG  		: 	out 		std_logic_vector(7 downto 0);
                
      -- DRAM ports
      DRAM_CLK		: 	out 		std_logic;
		DRAM_CKE 	: 	out 		std_logic;
      DRAM_ADDR	: 	out 		std_logic_vector(11 DOWNTO 0);
      DRAM_BA_1	: 	buffer 	std_logic;
		DRAM_BA_0 	: 	buffer 	std_logic;
      DRAM_CS_N 	: 	out 		std_logic; 
		DRAM_CAS_N 	: 	out 		std_logic; 
		DRAM_RAS_N 	: 	out 		std_logic; 
		DRAM_WE_N 	: 	out 		std_logic;
      DRAM_DQ 		: 	inout 	std_logic_vector(15 DOWNTO 0);
      DRAM_UDQM 	:	buffer 	std_logic; 
		DRAM_LDQM 	: 	buffer 	std_logic ;

		-- Flash Ports
		FL_ADDR :       out     std_logic_vector(21 downto 0);
		FL_DQ   :       inout   std_logic_vector(7 downto 0);
		FL_OE_N :       out     std_logic;
		FL_CE_N :       out     std_logic;
		FL_WE_N :       out     std_logic;
		FL_RST_N        :       out     std_logic;

		-- Ethernet Ports
		ENET_DATA	:	inout		std_logic_vector(15 downto 0);
		ENET_CS_N	:	out		std_logic;
		ENET_CMD		:	out		std_logic;
		ENET_INT		:	in			std_logic;
		ENET_RD_N	:	out		std_logic;
		ENET_WR_N	:	out		std_logic;
		ENET_RST_N	:	out		std_logic;
		ENET_CLK		:	out		std_logic;
		
		-- USB Ports
		OTG_CS_N		:	out		std_logic;
		OTG_RD_N		:	out		std_logic;
		OTG_WR_N		:	out		std_logic;
		OTG_RST_N	:	out		std_logic;
		OTG_ADDR		:	out		std_logic_vector(1 downto 0);
		OTG_DATA		:	inout		std_logic_vector(15 downto 0);
		OTG_INT0		:	in			std_logic;
		OTG_INT1		:	in			std_logic;
		OTG_FSPEED	:	out		std_logic;
		OTG_LSPEED	:	out		std_logic;
		OTG_DACK0_N	:	out		std_logic;
		OTG_DACK1_N	:	out		std_logic;
		OTG_DREQ0	:	out		std_logic;
		OTG_DREQ1	:	out		std_logic;
		
		-- UART
		UART_RXD		:	in			std_logic;
		UART_TXD		:	out		std_logic;
		
      -- CLOCK port
      CLOCK_50 	:	in  		std_logic;
					 
		-- Switch			
		SW				:	in 		std_logic_vector(0 downto 0);
		
      -- RESET key
      KEY 			:	in  		std_logic_vector(0 downto 0)
      );
end niosII_HomeAutomation;


-- Library Clause(s) (optional)
-- Use Clause(s) (optional)

architecture structure of niosII_HomeAutomation is

-- A component declaration declares the interface of an entity or
-- a design unit written in another language.  VHDL requires that
-- you declare a component if you do not intend to instantiate
-- an entity directly.  The component need not declare all the
-- generics and ports in the entity.  It may omit generics/ports
-- with default values.

component niosII_HomeAutomation_sys

        port
        (

			-- LCD ports
			LCD_E_from_the_lcd_0 : out std_logic;
         LCD_RS_from_the_lcd_0 : out std_logic;
         LCD_RW_from_the_lcd_0 : out std_logic;
         LCD_data_to_and_from_the_lcd_0 : inout std_logic_vector (7 downto 0);

         -- PLL ports
         altpll_0_c0_out : out std_logic;
         altpll_0_c1_out : out std_logic;
			altpll_0_c2_out : out std_logic;
         locked_from_the_altpll_0 : out std_logic;

         -- LEDS 
			out_port_from_the_green_leds : out std_logic_vector (7 downto 0);
					
         -- One more PLL port
         phasedone_from_the_altpll_0 : out std_logic;

         -- DRAM ports
         zs_addr_from_the_sdram_0 : out std_logic_vector (11 DOWNTO 0);
         zs_ba_from_the_sdram_0 : out std_logic_vector (1 DOWNTO 0);
         zs_cas_n_from_the_sdram_0 : out std_logic;
			zs_cke_from_the_sdram_0 : out std_logic;
         zs_cs_n_from_the_sdram_0 : out std_logic;
         zs_dq_to_and_from_the_sdram_0 : inout std_logic_vector (15 DOWNTO 0);
         zs_dqm_from_the_sdram_0 : out std_logic_vector (1 DOWNTO 0);
         zs_ras_n_from_the_sdram_0 : out std_logic;

			zs_we_n_from_the_sdram_0 : out std_logic;

			
		   -- the_DM9000A_0
		   ENET_CMD_from_the_DM9000A_0 : OUT STD_LOGIC;
		   ENET_CS_N_from_the_DM9000A_0 : OUT STD_LOGIC;
		   ENET_DATA_to_and_from_the_DM9000A_0 : INOUT STD_LOGIC_VECTOR (15 DOWNTO 0);
		   ENET_INT_to_the_DM9000A_0 : IN STD_LOGIC;
		   ENET_RD_N_from_the_DM9000A_0 : OUT STD_LOGIC;
		   ENET_RST_N_from_the_DM9000A_0 : OUT STD_LOGIC;
		   ENET_WR_N_from_the_DM9000A_0 : OUT STD_LOGIC;
			
			-- the_tri_state_bridge_0_avalon_slave
			address_to_the_cfi_flash_0 : OUT STD_LOGIC_VECTOR (21 DOWNTO 0);
		   data_to_and_from_the_cfi_flash_0 : INOUT STD_LOGIC_VECTOR (7 DOWNTO 0);
			read_n_to_the_cfi_flash_0 : OUT STD_LOGIC;
			select_n_to_the_cfi_flash_0 : OUT STD_LOGIC;
			write_n_to_the_cfi_flash_0 : OUT STD_LOGIC;
			
			-- the_ISP1362
			avs_hc_export_OTG_ADDR_from_the_ISP1362 : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
			avs_hc_export_OTG_CS_N_from_the_ISP1362 : OUT STD_LOGIC;
			avs_hc_export_OTG_DATA_to_and_from_the_ISP1362 : INOUT STD_LOGIC_VECTOR (15 DOWNTO 0);
			avs_hc_export_OTG_INT0_to_the_ISP1362 : IN STD_LOGIC;
			avs_dc_export_OTG_INT1_to_the_ISP1362 : IN STD_LOGIC;
			avs_hc_export_OTG_RD_N_from_the_ISP1362 : OUT STD_LOGIC;
			avs_hc_export_OTG_RST_N_from_the_ISP1362 : OUT STD_LOGIC;
			avs_hc_export_OTG_WR_N_from_the_ISP1362 : OUT STD_LOGIC;
			
			-- the_uart_0
         rxd_to_the_uart_0 : IN STD_LOGIC;
         txd_from_the_uart_0 : OUT STD_LOGIC;
						  
         -- CLOCK port
         clk_0           : in  std_logic;
					 
			-- SWITCH to change message
			in_port_to_the_switch : in std_logic;
			
         -- RESET Key
         reset_n : in  std_logic
        );
end component;

        signal BA : std_logic_vector (1 downto 0);
        signal DQM : std_logic_vector (1 downto 0);

		  -- I have no use for these signals from the pll so I just
		  -- connect them with signals
		  -- I specifically asked the PLL to omit these signals and yet
		  -- they are generated anyway
		  
        signal pll_c1:std_logic;
		  signal pll_c2:std_logic;
        signal pll_locked:std_logic;
        signal pll_phase: std_logic;
		  
		  signal otg_rst_n_tmp: std_logic;
		  

begin
		-- The SOPC-created IP does not turn the LCD on at all.
      -- I've explicitly turned it on with the next LOC. Note that LCD_ON is the
      -- VCC power line not the programmaticly controlled version that is bit-banged.
        LCD_ON <= '1';

		-- I need to make the mapping between the BA and DQM lines of the niosII_system
		-- and the physical DRAM 
        DRAM_BA_1 <= BA(1);
        DRAM_BA_0 <= BA(0);
        DRAM_UDQM <= DQM(1);
        DRAM_LDQM <= DQM(0);

        -- Component Instantiation Statement (optional)
        NIOSII : niosII_HomeAutomation_sys port map ( 
			  LCD_E_from_the_lcd_0 => LCD_EN, 
			  LCD_RS_from_the_lcd_0 => LCD_RS, 
			  LCD_RW_from_the_lcd_0 => LCD_RW, 
			  LCD_data_to_and_from_the_lcd_0 => LCD_DATA,
			  altpll_0_c0_out => DRAM_CLK, 
			  altpll_0_c1_out => pll_c1, 
			  altpll_0_c2_out => pll_c2, 
			  locked_from_the_altpll_0 => pll_locked,
			  out_port_from_the_green_leds => LEDG, 
			  phasedone_from_the_altpll_0 => pll_phase,
			  zs_addr_from_the_sdram_0 => DRAM_ADDR, 
			  zs_ba_from_the_sdram_0 => BA, 
			  zs_cas_n_from_the_sdram_0 => DRAM_CAS_N, 
			  zs_cke_from_the_sdram_0 => DRAM_CKE, 
			  zs_cs_n_from_the_sdram_0 => DRAM_CS_N,
			  zs_dq_to_and_from_the_sdram_0 => DRAM_DQ, 
			  zs_dqm_from_the_sdram_0 => DQM, 
			  zs_ras_n_from_the_sdram_0 => DRAM_RAS_N, 
			  zs_we_n_from_the_sdram_0 => DRAM_WE_N,
			  address_to_the_cfi_flash_0 => FL_ADDR,
			  data_to_and_from_the_cfi_flash_0 => FL_DQ,
			  read_n_to_the_cfi_flash_0 => FL_OE_N,
			  select_n_to_the_cfi_flash_0 => FL_CE_N,
			  write_n_to_the_cfi_flash_0 => FL_WE_N,
			  ENET_CMD_from_the_DM9000A_0 => ENET_CMD,
		     ENET_CS_N_from_the_DM9000A_0 => ENET_CS_N,
		     ENET_DATA_to_and_from_the_DM9000A_0 => ENET_DATA,
		     ENET_INT_to_the_DM9000A_0 => ENET_INT,
		     ENET_RD_N_from_the_DM9000A_0 => ENET_RD_N,
		     ENET_RST_N_from_the_DM9000A_0 => ENET_RST_N,
		     ENET_WR_N_from_the_DM9000A_0 => ENET_WR_N,
			  
			  avs_hc_export_OTG_ADDR_from_the_ISP1362 => OTG_ADDR,
			  avs_hc_export_OTG_CS_N_from_the_ISP1362 => OTG_CS_N,
			  avs_hc_export_OTG_DATA_to_and_from_the_ISP1362 => OTG_DATA,
			  avs_hc_export_OTG_RD_N_from_the_ISP1362 => OTG_RD_N,
			  avs_hc_export_OTG_RST_N_from_the_ISP1362 => OTG_RST_N,
			  avs_hc_export_OTG_WR_N_from_the_ISP1362 => OTG_WR_N,
			  avs_hc_export_OTG_INT0_to_the_ISP1362 => OTG_INT0,
			  avs_dc_export_OTG_INT1_to_the_ISP1362 => OTG_INT1,
			
			  -- the_uart_0
           rxd_to_the_uart_0 => UART_RXD,
           txd_from_the_uart_0 => UART_TXD,
						  
			  clk_0 => CLOCK_50, 
			  in_port_to_the_switch => SW(0), 
			  reset_n => KEY(0)
			);
			
			FL_RST_N <= '1';
			ENET_CLK <= pll_c2;
			
			-- USB Speed
			OTG_FSPEED <= 'Z';
			OTG_LSPEED <= 'Z';
			
end structure;
		  
