/**
 * X10 Driver
 *
 *  Authors: Tarek Kaddoura
 *           Jigar Nahar
 **/

#include "X10Driver.h"

int InitializeX10Device(X10 *x10, int port) {
    int atl_skip=0xFFFFFFFE;
    int atl_last=0x00000001;
    int atl_blk_size=64;
    int atl_cnt=1;
    int atl_timeout=200;
    
    x10->write_ep = X10_WRITE_EP;
    x10->read_ep = X10_READ_EP;
    x10->port = port;
    x10->config = X10_CONFIG;
    
    USBSetupATLParameters(atl_skip, atl_last, atl_blk_size, atl_cnt, atl_timeout);
    InitializeUSBDevice(&(x10->usbDevice), port);
    
    return USBDeviceSetConfig(&(x10->usbDevice), X10_CONFIG); // should return 0
}

int X10SendCommand(X10 *x10, int *payload, int plength, int pbytes) {
    unsigned long int_skip=0xFFFFFFFE;
    unsigned long int_last=0x00000001;
    unsigned int  int_blk_size=64;
    int freq = 0x0;
    
    USBSetupINTParameters(int_skip, int_last, int_blk_size);
    return USBDeviceInterruptWrite(&(x10->usbDevice), x10->write_ep, payload, plength, pbytes, freq); // should return > 0
}
            
int X10Send(X10 *x10, unsigned char cmd, char hcode,  int ucode) {
    // Command is always 5 bytes long
    int *cmd_r = generateCommand(cmd, hcode, ucode);
    return X10SendCommand(x10, cmd_r, 3, 5);
}

int *generateCommand(unsigned char cmd, char hcode, int ucode) {
    int unitCodes[] = {
        0x0000, 0x0010, 0x0008, 0x0018, 
        0x0040, 0x0050, 0x0048, 0x0058, 
        0x0400, 0x0410, 0x0408, 0x0418, 
        0x0440, 0x0450, 0x0448, 0x0458
    };
    
    int houseCodes[] = {
        0x060, 0x070, 0x040, 0x050,
        0x080, 0x090, 0x0A0, 0x0B0,
        0x0E0, 0x0F0, 0x0C0, 0x0D0,
        0x000, 0x010, 0x020, 0x030
    };
    
    unsigned char buf[5];
    unsigned char unit, house;
    static int buf_r[3];
    
    // Force the house code to start at 0 when it starts at ASCII_A
    hcode = hcode - ASCII_A;
    if (hcode < 0 || hcode > 15)
        return 0;
    if (ucode < 0 || ucode > 15)
        return 0;
        
    if (ucode > 0)
        unit = unitCodes[ucode-1];
    else
        unit = 0;
        
    house = houseCodes[(int) hcode];
        
    buf[0] = CMD_PFX;
    buf[1] = (unit >> 8) | house;
    buf[2] = 0x0FF & ~buf[1];
    
    if (ucode == 0) {
        if (cmd == CMD_ON)
            buf[3] = 0x90;
        else
            buf[3] = 0x80;
    }
    else 
        buf[3] = (unit & 0x0FF) | cmd;
    
    buf[4] = 0x0FF & ~buf[3];
    
    buf_r[0] = buf[0] | (buf[1] << 8);
    buf_r[1] = buf[2] | (buf[3] << 8);
    buf_r[2] = buf[4];
    
    return buf_r;
}

