/*
 * Header file for X10 Driver
 */

#ifndef X10DRIVER_H_
#define X10DRIVER_H_

#include "usb/Framework.h"

/*
 * Sturcture to hold the X10 device
 */
typedef struct {
    USBDevice usbDevice;
    int write_ep;
    int read_ep;
    int config;
    int port;
} X10;

#define X10_WRITE_EP 0x02 // write endpoint
#define X10_READ_EP 0x81 // read endpoint
#define X10_CONFIG 1 // Configuration value
#define X10_PORT 2 // Port number

// X10 commands
#define CMD_PFX 0x020
#define CMD_ON 0x0
#define CMD_OFF 0x20
#define CMD_BRIGHT 0x088

#define ASCII_A 65

/**
 * X10SendCommand
 * Send the @payload to @x10
 * @param x10 The x10 device structure
 * @param payload The payload to send as an integer array
 * @param plength The number of elements in @payload
 * @param pbytes The size of @payload in bytes
 * @return 0 on success.
 */
int X10SendCommand(X10 *x10, int *payload, int plength, int pbytes);

/**
 * InitializeX10Device
 * Initialize the X10 device with port @port
 * @return 0 on success.
 */
int InitializeX10Device(X10 *x10, int port);

/**
 * X10Send()
 * 
 * Turn an appliance on or off with 
 * house code @hcode and unit code @ucode
 * 
 * @param x10 The X10 device structure
 * @param cmd The command to send. Can be either CMD_ON or CMD_OFF.
 * @param hcode The house code of the appliance [A-P]
 * @param ucode The unit code of the appliance [1-16]
 * @return 0 on success
 */
int X10Send(X10 *x10, unsigned char cmd, char hcode,  int ucode);

/**
 * generateCommand()
 * 
 * Generate the payload sequence for the command @cmd
 * with house code @hcode and unit code @ucode
 * 
 * @param cmd The command to send. Can be either CMD_ON or CMD_OFF.
 * @param hcode The house code of the appliance [A-P]
 * @param ucode The unit code of the appliance [1-16]
 * @return The generated payload
 */
int *generateCommand(unsigned char cmd, char hcode, int ucode);

#endif /*X10DRIVER_H_*/
