/**
 *
 * USB Framework
 *
 * Authors: Tarek Kaddoura
 *          Jigar Nahar
 **/
 
#include "usb/HAL4D13.h"
#include "usb/isa290.h"
#include "usb/reg.h"
#include "usb/buf_man.h"
#include "usb/port.h"
#include "usb/usb.h"
#include "usb/ptd.h"
#include "usb/cheeyu.h"
#include "usb/ISP1362_HAL.h"
#include "Framework.h"

void InitializeUSB() {
  	w16(HcReset,0x00F6);//reset      
  	reset_usb();//config
    
    // Initialize USB ATL Parameters for control packets
    w16(HcControl,0x6c0);
    w16(HcUpInt,0x1a9);

    w16(HcBufStatus,0x00);
}

void USBSetupATLParameters(int atl_skip, int atl_last, int atl_blk_size, int atl_cnt, int atl_timeout) {
    //Setup ATL Parameters
    w32(HcATLSkip, atl_skip);
    w32(HcATLLast, atl_last);
    w16(HcATLBlkSize, atl_blk_size);
    w16(HcATLThrsCnt, atl_cnt);
    w16(HcATLTimeOut, atl_timeout);
}

int InitializeUSBDevice(USBDevice *usbDevice, int port) {
	int status, address;
    
    address = -1;
    
    // set the ports as operational and enable them
    set_operational();
    enable_port();
    usbDevice->port = port;
    
    status=assign_address(1,2,0);
    
    // is port 2 active and assigned?
    // assign an address if it is
    if (status&0x1 != 0)
        address = port;
    
    usbDevice->address = address;
    return address;
}

int USBDeviceSetConfig(USBDevice *usbDevice, int config) {
    int mycode;
    
    mycode = set_config(usbDevice->address, config);
    return mycode;
}

int USBDeviceGetDeviceDescriptor(USBDevice *usbDevice) {
    int rbuf[128], mycode;
    char *rbufc;
    
    if (usbDevice->address < 0)
        return -1;
        
    mycode = get_control(rbuf, usbDevice->address, 'D', 0, usbDevice->port);   
    if (mycode != 0x0300)
        return 0;
    
    // else fill out the descriptor with the rbuf
    USBDeviceDescriptor *desc = &(usbDevice->usbDeviceDescriptor);
    desc->bLength = rbuf[0]&0xFF;
    desc->bDescriptorType = rbuf[0] >> 8;
    desc->bcdUSB = rbuf[1];
    desc->bDeviceClass = rbuf[2]&0xFF;
    desc->bDeviceSubClass = rbuf[2] >> 8;
    desc->bDeviceProtocol = rbuf[3]&0xFF;
    desc->bMaxPacketSize0 = rbuf[3] >> 8;
    desc->idVendor = rbuf[4];
    desc->idProduct = rbuf[5];
    desc->bcdDevice = rbuf[6];
    desc->iManufacturer = rbuf[7] & 0xFF;
    desc->iProduct = rbuf[7] >> 8;
    desc->iSerialNumber = rbuf[8] & 0xFF;
    desc->bNumConfigurations = rbuf[8] >> 8;
    
    return 1;
}

int USBDeviceInterruptWrite(USBDevice *usbDevice, int endpoint, int *payload, int psize, int pbytes, int freq) {
     unsigned int cbuf[128], rbuf[128];
     
     // Create the Interrupt PTD
     make_int_ptd(cbuf, OUT, endpoint, pbytes, 0, usbDevice->address, usbDevice->port, freq);
     array_app(cbuf+4, payload, psize);
     // send it
     return send_int(cbuf, rbuf);
}

int USBSetupINTParameters(int int_skip, int int_last, int int_blk_size) {
     //Setup Int Parameters
     w32(HcIntSkip,int_skip);
     w32(HcIntLast,int_last);
     w16(HcIntBlkSize,int_blk_size);   
}
